export declare type AsyncState<R> = {
    loading: boolean;
    error: Error | undefined;
    result: R | undefined;
};
declare type SetLoading<R> = (asyncState: AsyncState<R>) => AsyncState<R>;
declare type SetResult<R> = (result: R, asyncState: AsyncState<R>) => AsyncState<R>;
declare type SetError<R> = (error: Error, asyncState: AsyncState<R>) => AsyncState<R>;
declare type MaybePromise<T> = Promise<T> | T;
export declare type UseAsyncOptionsNormalized<R> = {
    initialState: () => AsyncState<R>;
    executeOnMount: boolean;
    executeOnUpdate: boolean;
    setLoading: SetLoading<R>;
    setResult: SetResult<R>;
    setError: SetError<R>;
};
export declare type UseAsyncOptions<R> = Partial<UseAsyncOptionsNormalized<R>> | undefined | null;
export declare type UseAsyncReturn<R, Args extends any[] = never> = AsyncState<R> & {
    set: (value: AsyncState<R>) => void;
    reset: () => void;
    execute: (...args: Args) => Promise<R>;
    currentPromise: Promise<R> | null;
};
export declare function useAsync<R, Args extends any[]>(asyncFunction: () => Promise<R>, params: Args, options?: UseAsyncOptions<R>): UseAsyncReturn<R, Args>;
export declare function useAsync<R, Args extends any[]>(asyncFunction: (...args: Args) => Promise<R>, params: Args, options?: UseAsyncOptions<R>): UseAsyncReturn<R, Args>;
declare type AddArg<H, T extends any[]> = ((h: H, ...t: T) => void) extends ((...r: infer R) => void) ? R : never;
export declare const useAsyncAbortable: <R, Args extends any[]>(asyncFunction: (...args: AddArg<AbortSignal, Args>) => Promise<R>, params: Args, options?: UseAsyncOptions<R>) => UseAsyncReturn<R, Args>;
export declare const useAsyncCallback: <R, Args extends any[]>(asyncFunction: (...args: Args) => MaybePromise<R>) => UseAsyncReturn<R, Args>;
export {};
